import requests

LOGIN_URL = "https://secure.nicovideo.jp/secure/login"
MYPAGE_URL = "http://www.nicovideo.jp/my/top"


class LoginError(Exception):
    pass


class Session(object):
    def __init__(self, session=None):
        if not session:
            session = requests.Session()
        self._session = session

    def get(self, *args, **kwargs):
        return self._session.get(*args, **kwargs)

    def post(self, *args, **kwargs):
        return self._session.post(*args, **kwargs)

    def is_login(self, response=None):
        if not response:
            response = self.get(MYPAGE_URL)
        # 0: Logout
        # 1: Login as free user
        # 3: Login as premium user
        return not response.headers["x-niconico-authflag"] == "0"

    def login(self, mail=None, password=None):
        if mail:
            self.mail = mail
        if password:
            self.password = password
        response = self.post(LOGIN_URL, data={
            "mail": self.mail,
            "password": self.password,
        })
        if not self.is_login(response):
            raise LoginError(
                "mail: \"{mail}\", password: \"{password}\"".format(
                    mail=self.mail,
                    password=self.password,
                )
            )

    def logout(self):
        self._session = requests.Session()


def login(mail, password):
    session = Session()
    session.login(mail, password)
    return session



from argparse import ArgumentParser
import os
import re
import sys

from kishimenpy import api, auth

VIDEO_URL_PATTERN = re.compile(r"http://www\.nicovideo\.jp/watch/(\w+)")
BLOCK_SIZE = 1024 * 10  # 10KB


def cmd_download(args):
    mail = args.mail
    password = args.password
    if not (mail and password):
        # TODO: raw input?
        print("Please specify mail and password")
        sys.exit(1)

    dest = os.path.expanduser(args.dest)
    if not os.path.exists(dest):
        os.mkdir(dest)

    session = auth.login(mail, password)

    for video_url in args.video_urls:
        download(session, video_url, dest, args.retry)


def get_video_id(video_url):
    m = VIDEO_URL_PATTERN.match(video_url)
    return m.group(1)


def download(session, video_url, dest, retry):
    video_id = get_video_id(video_url)
    thumb_info = api.get_thumb_info(video_id)
    title = thumb_info["title"]
    movie_type = thumb_info["movie_type"]
    if movie_type == "swf":
        ext = "flv"
    else:
        ext = movie_type

    filename = "{video_id}_{title}.{ext}".format(
        video_id=video_id, title=title, ext=ext
    )
    # TODO: more safe filename
    filename = filename.replace(" ", "")
    filename = filename.replace("　", "")
    filename = filename.replace("/", "_")
    filepath = os.path.join(dest, filename)

    if os.path.exists(filepath):
        print("Already exists: {filename}".format(filename=filename))
        return

    with open(filepath, "wb") as fp:
        print("Directory: {dest}".format(dest=dest))
        print("Filename : {filename}".format(filename=filename))
        print("Downloading video data ...")
        sys.stdout.flush()

        retry_count = 0
        while retry_count <= retry:
            try:
                flv = api.get_flv(session, video_id)
                flv_url = flv["url"]
                # Access once and cache
                session.get(video_url)
                response = session.get(flv_url, stream=True)
                if not (
                    response.status_code == 200 or  # OK
                    response.status_code == 206     # Partial Content
                ):
                    # TODO: Forbidden
                    raise auth.LoginError(response.status_code)
                while True:
                    video_data = response.raw.read(BLOCK_SIZE)
                    if not video_data:
                        break
                    fp.write(video_data)
                    # TODO: Show progress bar
            except (
                api.VideoDeleted,
                api.VideoNotFound,
                api.UnknownError,
            ) as e:
                print(e)
                retry_count += 1
            except (
                auth.LoginError,
            ) as e:
                print(e)
                session.login()
                retry_count += 1
            else:
                break


def create_parser():
    parser = ArgumentParser()
    subparsers = parser.add_subparsers()
    download_parser = subparsers.add_parser("download")
    download_parser.add_argument("-m", "--mail", dest="mail")
    download_parser.add_argument("-p", "--password", dest="password")
    download_parser.add_argument("-d", "--dest", dest="dest", default=".")
    download_parser.add_argument("-r", "--retry", dest="retry", default=5)
    download_parser.add_argument("video_urls", nargs="*")
    download_parser.set_defaults(func=cmd_download)
    return parser


def main():
    parser = create_parser()
    args = parser.parse_args()

    if hasattr(args, "func"):
        args.func(args)



from xml.etree import ElementTree
from urllib.parse import unquote

import requests

from kishimenpy import auth

GET_THUMB_INFO_URL = "http://ext.nicovideo.jp/api/getthumbinfo/{video_id}"
THUMB_URL = "http://ext.nicovideo.jp/thumb/{video_id}"
GET_FLV_URL = "http://flapi.nicovideo.jp/api/getflv/{video_id}"
GET_MARQUEE_URL = "http://flapi.nicovideo.jp/api/getmarquee?"
GET_RELATION_URL = (
    "http://flapi.nicovideo.jp/api/getrelation"
    "?page={page}"
    "&sort={sort}"
    "&order={order}"
    "&video={video_id}"
)
MSG_URL = "http://msg.nicovideo.jp/{video_id}/api"


class VideoDeleted(Exception):
    pass


class VideoNotFound(Exception):
    pass


class UnknownError(Exception):
    pass


class ThumbnailInfo(dict):
    def __init__(self, element):
        self.element = element
        for e in element.getchildren():
            if e.tag == "tags":
                self.__setitem__(e.tag, [
                    tag.text
                    for tag in e.getchildren()
                ])
            else:
                self.__setitem__(e.tag, e.text)


def get_thumb_info(video_id):
    res = requests.get(GET_THUMB_INFO_URL.format(video_id=video_id))

    root = ElementTree.fromstring(res.text)
    status = root.get("status")
    if not status == "ok":
        error = root.getchildren()[0]
        code, description = error.getchildren()
        if code.text == "DELETED":
            raise VideoDeleted(description.text)
        if code.text == "NOT_FOUND":
            raise VideoNotFound(description.text)
        else:
            raise UnknownError(
                "code: {code}, description: {description}".format(
                    code=code.text,
                    description=description.text,
                )
            )
    return ThumbnailInfo(root.getchildren()[0])


def get_flv(session, video_id):
    res = session.get(GET_FLV_URL.format(video_id=video_id))
    flv = {
        p.split("=")[0]: unquote(p.split("=")[1])
        for p in res.text.split("&")
    }
    if "closed" in flv and flv["closed"] == "1":
        raise auth.LoginError(flv)
    else:
        return flv



import os

from setuptools import setup, find_packages

version = '0.1'

here = os.path.abspath(os.path.dirname(__file__))
try:
    with open(os.path.join(here, 'README.txt')) as f:
        README = f.read()
    with open(os.path.join(here, 'CHANGES.txt')) as f:
        CHANGES = f.read()
except IOError:
    README = CHANGES = ''

long_description = (
    README
    + '\n'
    + CHANGES
    + '\n'
)

requires = [
    'requests',
]

entry_points = {
    'console_scripts': [
        'kishimenpy = kishimenpy.commands:main',
    ],
}

setup(
    name='kishimenpy',
    version=version,
    description="The niconico Downloader",
    long_description=long_description,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.3",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
    ],
    keywords='',
    author='Jiro Nejime',
    author_email='neji@drillbits.jp',
    url='https://github.com/drillbits/kishimenpy',
    license='MIT',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,
    zip_safe=False,
    install_requires=requires,
    entry_points=entry_points,
)
